(*----------------------------------------------------------------------------*
 *  NVIDIA Cg Delphi demo                                                     *
 *  This demo is based on "runtime_ogl\cgGL_vertex_example" sample from       *
 *  NVIDIA Cg toolkit Beta 1                                                  *
 *                                                                            *
 * Portions created by NVidia                                                 *
 *  Copyright (c) 2002 NVIDIA                                                 *
 *                                                                            *
 * Portions created by me                                                     *
 *  Copyright (c) 2002 by Alexey Barkovoy                                     *
 *  E-Mail: clootie@reactor.ru                                                *
 *                                                                            *
 *  Modified: 20-Jun-2002                                                     *
 *                                                                            *
 *  Latest version can be downloaded from:                                    *
 *     http://clootie.narod.ru                                                *
 *----------------------------------------------------------------------------*)

program cgGL_vertex_example;

uses
  Windows,
  Messages,
  SysUtils,
  OpenGL12,
  cg,
  cgGL;

const
  WND_TITLE = 'NVIDIA Cg OpenGL sample *** http:/clootie.narod.ru';
  FPS_TIMER = 1;                     // Timer to calculate FPS
  FPS_INTERVAL = 1000;               // Calculate FPS every 1000 ms

(******************************************************************************)
(*** Static Data                                                            ***)
(******************************************************************************)

const
  TextureRes: Integer = 512;

var
  Context: PcgContext = nil;
  ProgramIter: PcgProgramIter = nil;
  KdBind: PcgBindIter = nil;
  ModelViewProjBind: PcgBindIter = nil;
  TestColorBind: PcgBindIter = nil;

var
  CGVertProg: PChar =
 ''#10 +
 'struct appdata'#10 +
 ' {'#10 +
 '  float4 position : POSITION;'#10 +
 '  float3 normal : NORMAL;'#10 +
 '  float3 color : DIFFUSE;'#10 +
 '  float3 TestColor : SPECULAR;'#10 +
 ' };'#10 +
 ''#10 +
 'struct vfconn'#10 +
 ' {'#10 +
 '  float4 HPOS : POSITION;'#10 +
 '  float4 COL0 : COLOR0;'#10 +
 ' };'#10 +
 ''#10 +
 'vfconn main(appdata IN,'#10 +
 '            uniform float4 Kd,'#10 +
 '            uniform float4x4 ModelViewProj)'#10 +
 ' {'#10 +
 '  vfconn OUT;'#10 +
 ''#10 +
 '  OUT.HPOS = mul(ModelViewProj, IN.position);'#10 +
 ''#10 +
 '  OUT.COL0.xyz = Kd.xyz * IN.TestColor.xyz;'#10 +
 '  OUT.COL0.w = 1.0;'#10 +
 ''#10 +
 '  return OUT;'#10 +
 ' } // main'#0;

var
  LightDiffuse: array [0..3] of GLfloat = (1.0, 0.0, 0.0, 1.0);
  LightPosition: array [0..3] of GLfloat = (1.0, 1.0, 1.0, 0.0);

  CubeNormals: array [0..5, 0..2] of GLfloat =
   (
    (-1.0, 0.0, 0.0), (0.0, 1.0, 0.0), (1.0, 0.0, 0.0),
    (0.0, -1.0, 0.0), (0.0, 0.0, 1.0), (0.0, 0.0, -1.0)
   );

  CubeFaces: array [0..5, 0..3] of GLint  =
   (
    (0, 1, 2, 3), (3, 2, 6, 7), (7, 6, 5, 4),
    (4, 5, 1, 0), (5, 6, 2, 1), (7, 4, 0, 3)
   );

type
  TCube = array [0..7, 0..2] of GLfloat;
var
  CubeVertices: TCube;

var
  h_Wnd  : HWND;                     // Global window handle
  h_DC   : HDC;                      // Global device context
  h_RC   : HGLRC;                    // OpenGL rendering context
  keys : Array[0..255] of Boolean;   // Holds keystrokes
  FPSCount : Integer = 0;            // Counter for FPS
  ElapsedTime : Integer;             // Elapsed time between frames

  // User vaiables
  xSpeed, ySpeed : glFloat;

{$R *.RES}

{------------------------------------------------------------------}
{  Function to convert int to string. (No sysutils = smaller EXE)  }
{------------------------------------------------------------------}
function IntToStr(Num : Integer) : String;  // using SysUtils increase file size by 100K
begin
  Str(Num, result);
end;

//******************************************************************************
//******************************************************************************
//******************************************************************************

//function MATRIX_INDEX(i, j: Integer): Integer; begin Result:= (j + i * 4); end;

procedure DrawCube;
var
  i: Integer;
  Ret: TcgError;
//  TexUnit: GLenum; // = GL_TEXTURE0_ARB;
begin
//  TexUnit:= GL_TEXTURE0_ARB;

  Assert(@glMultiTexCoord2fARB <> nil);
  if (@glMultiTexCoord2fARB = nil) then Exit;

  Ret := cgGLBindProgram(ProgramIter);
  Assert(Ret = cgNoError);

  (*
   * Bind various uniform parameters including the ModelViewProjection
   * matrix for transforming the incoming position into HPOS.
   *)
  if (KdBind <> nil) then
    cgGLBindUniform4f(ProgramIter, KdBind, 1.0, 1.0, 0.0, 1.0);

  (* Bind the concatenate modelview and projection matrices *)
  if (ModelViewProjBind <> nil) then
    cgGLBindUniformStateMatrix(ProgramIter,
                               ModelViewProjBind,
                               cgGLModelViewProjectionMatrix,
                               cgGLMatrixIdentity);

  cgGLEnableProgramType(cgVertexProfile);


  (*
   * Create cube with per-vertex varying attributes
   *)
  for i:= 0 to 5 do
  begin
    glBegin(GL_QUADS);

    glNormal3fv(@CubeNormals[i][0]);
    cgGLBindVarying3f(ProgramIter, TestColorBind, 1.0, 0.0, 0.0);
    glVertex3fv(@CubeVertices[CubeFaces[i][0]][0]);

    cgGLBindVarying3f(ProgramIter, TestColorBind, 0.0, 1.0, 0.0);
    glVertex3fv(@CubeVertices[CubeFaces[i][1]][0]);

    cgGLBindVarying3f(ProgramIter, TestColorBind, 0.0, 0.0, 1.0);
    glVertex3fv(@CubeVertices[CubeFaces[i][2]][0]);

    cgGLBindVarying3f(ProgramIter, TestColorBind, 1.0, 1.0, 1.0);
    glVertex3fv(@CubeVertices[CubeFaces[i][3]][0]);

    glEnd;
  end;

  cgGLDisableProgramType(cgFragmentProfile);
end;

procedure InitializeCube(var v: TCube{ GLfloat v[8][3]});
begin
  (* Setup cube vertex data. *)
  v[0][0] := -1; v[1][0] := -1; v[2][0] := -1; v[3][0] := -1;
  v[4][0] :=  1; v[5][0] :=  1; v[6][0] :=  1; v[7][0] :=  1;
  v[0][1] := -1; v[1][1] := -1; v[4][1] := -1; v[5][1] := -1;
  v[2][1] :=  1; v[3][1] :=  1; v[6][1] :=  1; v[7][1] :=  1;
  v[0][2] :=  1; v[3][2] :=  1; v[4][2] :=  1; v[7][2] :=  1;
  v[1][2] := -1; v[2][2] := -1; v[5][2] := -1; v[6][2] := -1;
end;


{------------------------------------------------------------------}
{  Function to draw the actual scene                               }
{------------------------------------------------------------------}
procedure glDraw();
begin
  glClear(GL_COLOR_BUFFER_BIT or GL_DEPTH_BUFFER_BIT);    // Clear The Screen And The Depth Buffer
  DrawCube;
end;


{------------------------------------------------------------------}
{  Initialise OpenGL                                               }
{------------------------------------------------------------------}
procedure glInit();
var
  Ret: TcgError;
  ProgId: GLuint; // = 1;
  S: String;
begin
  (* Enable a single OpenGL light. *)
  glLightfv(GL_LIGHT0, GL_DIFFUSE, @LightDiffuse);
  glLightfv(GL_LIGHT0, GL_POSITION, @LightPosition);
  glEnable(GL_LIGHT0);
//#if 0
//  glEnable(GL_LIGHTING);
//#else }
  glDisable(GL_LIGHTING);
//#endif

  (* Use depth buffering for hidden surface elimination. *)
  glEnable(GL_DEPTH_TEST);

  ReadExtensions;
  Assert(@glMultiTexCoord2fARB <> nil);

//*****************************************************************


  (* Test cgContext creation *)
  Context := cgCreateContext;
  Assert(Context <> nil);

  (* Test adding source text to context *)
//Ret := cgAddProgramFromFile(Context, 'cgGL_vertex_example.cg', cgVertexProfile, nil);
  Ret := cgAddProgram(Context, CGVertProg, cgVertexProfile, nil);
  MessageBox(0, PChar(Format('LAST LISTING----%s----', [cgGetLastListing(Context)])),
    'Compiler output', mb_Ok or mb_IconInformation);
  Assert(Ret = cgNoError);

  (* Test getting program handles from a context by the program's name *)
  ProgramIter := cgProgramByName(Context, 'main');
  Assert(ProgramIter <> nil);

  // Show compilation results
  S:= cgGetProgramObjectCode(ProgramIter);
  if S = '' then Exit;
  MessageBox(0, PChar(Format('---- PROGRAM BEGIN ----'#10'%s---- PROGRAM END ----',
    [S])), 'Compiler output',
    mb_Ok or mb_IconInformation);

  if (ProgramIter <> nil) then
  begin
    ProgId := 1;

    Ret := cgGLLoadProgram(ProgramIter, ProgId);
    assert(Ret = cgNoError);

    KdBind := cgGetBindByName(ProgramIter, 'Kd');
    assert(KdBind <> nil);

    ModelViewProjBind := cgGetBindByName(ProgramIter, 'ModelViewProj');
    assert(ModelViewProjBind <> nil);

    TestColorBind := cgGetBindByName(ProgramIter, 'IN.TestColor');
    assert(TestColorBind <> nil);
  end;
end;


{------------------------------------------------------------------}
{  Handle window resize                                            }
{------------------------------------------------------------------}
procedure glResizeWnd(Width, Height : Integer);
begin
  if (Height = 0) then                // prevent divide by zero exception
    Height := 1;

  glViewport(0, 0, Width, Height);    // Set the viewport for the OpenGL window

  (* Setup the view of the cube. *)
  glMatrixMode(GL_PROJECTION);
  glLoadIdentity();
  gluPerspective( (* field of view in degree *) 40.0,
    (* aspect ratio *) 1.0,
    (* Z near *) 1.0, (* Z far *) 10.0);
  glMatrixMode(GL_MODELVIEW);
  glLoadIdentity();
  gluLookAt(0.0, 0.0, 5.0,  (* eye is at (0,0,5) *)
            0.0, 0.0, 0.0,  (* center is at (0,0,0) *)
            0.0, 1.0, 0.0);  (* up is in positive Y direction *)

  (* Adjust cube position to be asthetic angle. *)
  glTranslatef(0.0, 0.0, -1.0);
//#if 1
  glRotatef(60, 1.0, 0.0, 0.0);
  glRotatef(-20, 0.0, 0.0, 1.0);
//#endif

end;


{------------------------------------------------------------------}
{  Processes all the keystrokes                                    }
{------------------------------------------------------------------}
procedure ProcessKeys;
begin
  if (keys[VK_UP])    then xspeed := xspeed - 0.002;
  if (keys[VK_DOWN])  then xspeed := xspeed + 0.002;
  if (keys[VK_RIGHT]) then yspeed := yspeed + 0.002;
  if (keys[VK_LEFT])  then yspeed := yspeed - 0.002;
end;


{------------------------------------------------------------------}
{  Determines the applications response to the messages received  }
{------------------------------------------------------------------}
function WndProc(hWnd: HWND; Msg: UINT;  wParam: WPARAM;  lParam: LPARAM): LRESULT; stdcall;
begin
  Result := 0;
  case (Msg) of
    WM_CREATE:
      begin
        // Insert stuff you want executed when the program starts
      end;
    WM_CLOSE:
      begin
        PostQuitMessage(0);
        Result := 0
      end;
    WM_KEYDOWN:       // Set the pressed key (wparam) to equal true so we can check if its pressed
      begin
        keys[wParam] := True;
        Result := 0;
      end;
    WM_KEYUP:         // Set the released key (wparam) to equal false so we can check if its pressed
      begin
        keys[wParam] := False;
        Result := 0;
      end;
    WM_SIZE:          // Resize the window with the new width and height
      begin
        glResizeWnd(LOWORD(lParam),HIWORD(lParam));
        Result := 0;
      end;
    WM_TIMER :                     // Add code here for all timers to be used.
      begin
        if wParam = FPS_TIMER then
        begin
          FPSCount :=Round(FPSCount * 1000/FPS_INTERVAL);   // calculate to get per Second incase intercal is less or greater than 1 second
          SetWindowText(h_Wnd, PChar(WND_TITLE + '   [' + intToStr(FPSCount) + ' FPS]'));
          FPSCount := 0;
          Result := 0;
        end;
      end;
    else
      Result := DefWindowProc(hWnd, Msg, wParam, lParam);    // Default result if nothing happens
  end;
end;


{---------------------------------------------------------------------}
{  Properly destroys the window created at startup (no memory leaks)  }
{---------------------------------------------------------------------}
procedure glKillWnd(Fullscreen : Boolean);
begin
  if Assigned(KdBind) then
   cgFreeBindIter(KdBind);

  if Assigned(ModelViewProjBind) then
    cgFreeBindIter(ModelViewProjBind);

  if Assigned(TestColorBind) then
    cgFreeBindIter(TestColorBind);

  cgFreeProgramIter(ProgramIter);
  cgFreeContext(Context);

  cgCleanup;

//************************************************************  

  if Fullscreen then             // Change back to non fullscreen
  begin
    ChangeDisplaySettings(devmode(nil^), 0);
    ShowCursor(True);
  end;

  // Makes current rendering context not current, and releases the device
  // context that is used by the rendering context.
  if (not wglMakeCurrent(h_DC, 0)) then
    MessageBox(0, 'Release of DC and RC failed!', 'Error', MB_OK or MB_ICONERROR);

  // Attempts to delete the rendering context
  if (not wglDeleteContext(h_RC)) then
  begin
    MessageBox(0, 'Release of rendering context failed!', 'Error', MB_OK or MB_ICONERROR);
    h_RC := 0;
  end;

  // Attemps to release the device context
  if ((h_DC > 0) and (ReleaseDC(h_Wnd, h_DC) = 0)) then
  begin
    MessageBox(0, 'Release of device context failed!', 'Error', MB_OK or MB_ICONERROR);
    h_DC := 0;
  end;

  // Attempts to destroy the window
  if ((h_Wnd <> 0) and (not DestroyWindow(h_Wnd))) then
  begin
    MessageBox(0, 'Unable to destroy window!', 'Error', MB_OK or MB_ICONERROR);
    h_Wnd := 0;
  end;

  // Attempts to unregister the window class
  if (not UnRegisterClass('OpenGL', hInstance)) then
  begin
    MessageBox(0, 'Unable to unregister window class!', 'Error', MB_OK or MB_ICONERROR);
    hInstance := 0;
  end;
end;


{--------------------------------------------------------------------}
{  Creates the window and attaches a OpenGL rendering context to it  }
{--------------------------------------------------------------------}
function glCreateWnd(Width, Height : Integer; Fullscreen : Boolean; PixelDepth : Integer) : Boolean;
var
  wndClass : TWndClass;         // Window class
  dwStyle : DWORD;              // Window styles
  dwExStyle : DWORD;            // Extended window styles
  dmScreenSettings : DEVMODE;   // Screen settings (fullscreen, etc...)
  PixelFormat : GLuint;         // Settings for the OpenGL rendering
  h_Instance : HINST;           // Current instance
  pfd : TPIXELFORMATDESCRIPTOR;  // Settings for the OpenGL window
begin
  h_Instance := GetModuleHandle(nil);       //Grab An Instance For Our Window
  ZeroMemory(@wndClass, SizeOf(wndClass));  // Clear the window class structure

  with wndClass do                    // Set up the window class
  begin
    style         := CS_HREDRAW or    // Redraws entire window if length changes
                     CS_VREDRAW or    // Redraws entire window if height changes
                     CS_OWNDC;        // Unique device context for the window
    lpfnWndProc   := @WndProc;        // Set the window procedure to our func WndProc
    hInstance     := h_Instance;
    hCursor       := LoadCursor(0, IDC_ARROW);
    lpszClassName := 'OpenGL';
  end;

  if (RegisterClass(wndClass) = 0) then  // Attemp to register the window class
  begin
    MessageBox(0, 'Failed to register the window class!', 'Error', MB_OK or MB_ICONERROR);
    Result := False;
    Exit
  end;

  // Change to fullscreen if so desired
  if Fullscreen then
  begin
    ZeroMemory(@dmScreenSettings, SizeOf(dmScreenSettings));
    with dmScreenSettings do begin              // Set parameters for the screen setting
      dmSize       := SizeOf(dmScreenSettings);
      dmPelsWidth  := Width;                    // Window width
      dmPelsHeight := Height;                   // Window height
      dmBitsPerPel := PixelDepth;               // Window color depth
      dmFields     := DM_PELSWIDTH or DM_PELSHEIGHT or DM_BITSPERPEL;
    end;

    // Try to change screen mode to fullscreen
    if (ChangeDisplaySettings(dmScreenSettings, CDS_FULLSCREEN) = DISP_CHANGE_FAILED) then
    begin
      MessageBox(0, 'Unable to switch to fullscreen!', 'Error', MB_OK or MB_ICONERROR);
      Fullscreen := False;
    end;
  end;

  // If we are still in fullscreen then
  if (Fullscreen) then
  begin
    dwStyle := WS_POPUP or                // Creates a popup window
               WS_CLIPCHILDREN            // Doesn't draw within child windows
               or WS_CLIPSIBLINGS;        // Doesn't draw within sibling windows
    dwExStyle := WS_EX_APPWINDOW;         // Top level window
    ShowCursor(False);                    // Turn of the cursor (gets in the way)
  end
  else
  begin
    dwStyle := WS_OVERLAPPEDWINDOW or     // Creates an overlapping window
               WS_CLIPCHILDREN or         // Doesn't draw within child windows
               WS_CLIPSIBLINGS;           // Doesn't draw within sibling windows
    dwExStyle := WS_EX_APPWINDOW or       // Top level window
                 WS_EX_WINDOWEDGE;        // Border with a raised edge
  end;

  // Attempt to create the actual window
  h_Wnd := CreateWindowEx(dwExStyle,      // Extended window styles
                          'OpenGL',       // Class name
                          WND_TITLE,      // Window title (caption)
                          dwStyle,        // Window styles
                          0, 0,           // Window position
                          Width, Height,  // Size of window
                          0,              // No parent window
                          0,              // No menu
                          h_Instance,     // Instance
                          nil);           // Pass nothing to WM_CREATE
  if h_Wnd = 0 then
  begin
    glKillWnd(Fullscreen);                // Undo all the settings we've changed
    MessageBox(0, 'Unable to create window!', 'Error', MB_OK or MB_ICONERROR);
    Result := False;
    Exit;
  end;

  // Try to get a device context
  h_DC := GetDC(h_Wnd);
  if (h_DC = 0) then
  begin
    glKillWnd(Fullscreen);
    MessageBox(0, 'Unable to get a device context!', 'Error', MB_OK or MB_ICONERROR);
    Result := False;
    Exit;
  end;

  // Settings for the OpenGL window
  with pfd do
  begin
    nSize           := SizeOf(TPIXELFORMATDESCRIPTOR); // Size Of This Pixel Format Descriptor
    nVersion        := 1;                    // The version of this data structure
    dwFlags         := PFD_DRAW_TO_WINDOW    // Buffer supports drawing to window
                       or PFD_SUPPORT_OPENGL // Buffer supports OpenGL drawing
                       or PFD_DOUBLEBUFFER;  // Supports double buffering
    iPixelType      := PFD_TYPE_RGBA;        // RGBA color format
    cColorBits      := PixelDepth;           // OpenGL color depth
    cRedBits        := 0;                    // Number of red bitplanes
    cRedShift       := 0;                    // Shift count for red bitplanes
    cGreenBits      := 0;                    // Number of green bitplanes
    cGreenShift     := 0;                    // Shift count for green bitplanes
    cBlueBits       := 0;                    // Number of blue bitplanes
    cBlueShift      := 0;                    // Shift count for blue bitplanes
    cAlphaBits      := 0;                    // Not supported
    cAlphaShift     := 0;                    // Not supported
    cAccumBits      := 0;                    // No accumulation buffer
    cAccumRedBits   := 0;                    // Number of red bits in a-buffer
    cAccumGreenBits := 0;                    // Number of green bits in a-buffer
    cAccumBlueBits  := 0;                    // Number of blue bits in a-buffer
    cAccumAlphaBits := 0;                    // Number of alpha bits in a-buffer
    cDepthBits      := 16;                   // Specifies the depth of the depth buffer
    cStencilBits    := 0;                    // Turn off stencil buffer
    cAuxBuffers     := 0;                    // Not supported
    iLayerType      := PFD_MAIN_PLANE;       // Ignored
    bReserved       := 0;                    // Number of overlay and underlay planes
    dwLayerMask     := 0;                    // Ignored
    dwVisibleMask   := 0;                    // Transparent color of underlay plane
    dwDamageMask    := 0;                     // Ignored
  end;

  // Attempts to find the pixel format supported by a device context that is the best match to a given pixel format specification.
  PixelFormat := ChoosePixelFormat(h_DC, @pfd);
  if (PixelFormat = 0) then
  begin
    glKillWnd(Fullscreen);
    MessageBox(0, 'Unable to find a suitable pixel format', 'Error', MB_OK or MB_ICONERROR);
    Result := False;
    Exit;
  end;

  // Sets the specified device context's pixel format to the format specified by the PixelFormat.
  if (not SetPixelFormat(h_DC, PixelFormat, @pfd)) then
  begin
    glKillWnd(Fullscreen);
    MessageBox(0, 'Unable to set the pixel format', 'Error', MB_OK or MB_ICONERROR);
    Result := False;
    Exit;
  end;

  // Create a OpenGL rendering context
  h_RC := wglCreateContext(h_DC);
  if (h_RC = 0) then
  begin
    glKillWnd(Fullscreen);
    MessageBox(0, 'Unable to create an OpenGL rendering context', 'Error', MB_OK or MB_ICONERROR);
    Result := False;
    Exit;
  end;

  // Makes the specified OpenGL rendering context the calling thread's current rendering context
  if (not wglMakeCurrent(h_DC, h_RC)) then
  begin
    glKillWnd(Fullscreen);
    MessageBox(0, 'Unable to activate OpenGL rendering context', 'Error', MB_OK or MB_ICONERROR);
    Result := False;
    Exit;
  end;

  // Initializes the timer used to calculate the FPS
  SetTimer(h_Wnd, FPS_TIMER, FPS_INTERVAL, nil);

  // Settings to ensure that the window is the topmost window
  ShowWindow(h_Wnd, SW_SHOW);
  SetForegroundWindow(h_Wnd);
  SetFocus(h_Wnd);

  // Ensure the OpenGL window is resized properly
  glResizeWnd(Width, Height);
  glInit();

  Result := True;
end;


{--------------------------------------------------------------------}
{  Main message loop for the application                             }
{--------------------------------------------------------------------}
function WinMain(hInstance : HINST; hPrevInstance : HINST;
                 lpCmdLine : PChar; nCmdShow : Integer) : Integer; stdcall;
var
  msg : TMsg;
  finished : Boolean;
  DemoStart, LastTime : DWord;
begin
  finished := False;

  // Perform application initialization:
  if not glCreateWnd(800, 600, FALSE, 32) then
  begin
    Result := 0;
    Exit;
  end;

  DemoStart := GetTickCount();            // Get Time when demo started

  // Main message loop:
  while not finished do
  begin
    if (PeekMessage(msg, 0, 0, 0, PM_REMOVE)) then // Check if there is a message for this window
    begin
      if (msg.message = WM_QUIT) then     // If WM_QUIT message received then we are done
        finished := True
      else
      begin                               // Else translate and dispatch the message to this window
        TranslateMessage(msg);
        DispatchMessage(msg);
      end;
    end
    else
    begin
      Inc(FPSCount);                      // Increment FPS Counter

      LastTime :=ElapsedTime;
      ElapsedTime :=GetTickCount() - DemoStart;     // Calculate Elapsed Time
      ElapsedTime :=(Integer(LastTime) + ElapsedTime) DIV 2; // Average it out for smoother movement

      glDraw();                           // Draw the scene
      SwapBuffers(h_DC);                  // Display the scene

      if (keys[VK_ESCAPE]) then           // If user pressed ESC then set finised TRUE
        finished := True
      else
        ProcessKeys;                      // Check for any other key Pressed
    end;
  end;
  glKillWnd(FALSE);
  Result := msg.wParam;
end;


begin
  LoadOpenGL;
  InitializeCube(CubeVertices);
  WinMain(hInstance, hPrevInst, CmdLine, CmdShow);
end.

